/*
 * This software is the confidential and proprietary information of
 * Smardec. You shall not disclose or distribute such Confidential Information.
 * This software shall not be integrated, copied (partially or fully),
 * or reverse engineered into any other software, program, or code that is
 * not intended for Smardec. You shall use it only in accordance with the terms
 * of the license agreement you entered into with Smardec.
 *
 * Copyright (C) 2004-2006 Smardec. All rights reserved.
 *
 * http://www.smardec.com
 *
 */

package com.smardec.demo.j2native.demos;

import com.smardec.asc.hyperlink.HyperlinkUtil;
import com.smardec.asc.table.SortedTableHeaderMouseListener;
import com.smardec.asc.table.SortedTableHeaderRenderer;
import com.smardec.asc.table.SortedTableModel;
import com.smardec.demo.j2native.DemoableAdapter;
import com.smardec.demo.j2native.Util;
import com.smardec.demo.j2native.linux.usermanager.GroupManager;
import com.smardec.demo.j2native.linux.usermanager.UserManager;
import com.smardec.demo.j2native.linux.usermanager.info.GroupInfo;
import com.smardec.demo.j2native.linux.usermanager.info.UIGroupPropertiesDialog;
import com.smardec.demo.j2native.linux.usermanager.info.UIUserPropertiesDialog;
import com.smardec.demo.j2native.linux.usermanager.info.UserInfo;
import com.smardec.demo.j2native.linux.usermanager.j2n.InfoProvider;
import com.smardec.demo.j2native.tools.SysInfoUtil;

import javax.swing.*;
import javax.swing.border.TitledBorder;
import javax.swing.event.*;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableCellRenderer;
import java.awt.*;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;
import java.util.Vector;

public class LinUserManagerDemo extends DemoableAdapter {
    public final static String CONSTRAINT_LIN_USER_MANAGER = "LinUserManager";
    public final static String CAPTION_LIN_USER_MANAGER = "User manager";
    public final static String INTRO_LIN_USER_MANAGER = "<html><head>" + css + "</head><body>" +
            "<p align=left color=black>User info reflects information about system users. To do that the program chooses data from <i>/etc/passwd</i> and <i>/etc/shadow</i> files using system API. (NOTE: program works in Read-Only mode. No changes will be made under any circumstances.)</p>" +
            "</body></html>";
    public final static String URL_API_REFERENCE = "viewer.html";
    public final static String API_REFERENCE_PATH = "demo-resources/api-reference/mansec.html";

    public final static String SRC_LIN_USER_MANAGER = "<html><head>" + css + "</head><body>" +
            "<p align=left color=black>If we address to man pages we will find out that to receive the information on users of system it is necessary to use getspwd() function. This function fills spwd structure with the information necessary for us. Lets define spwd structure in terms of J2Native:" +
            "<br><pre>" +
            "public class Spwd extends Structure {\n" +
            "    public AnsiString sp_namp;   /* Login name.  */\n" +
            "    public AnsiString sp_pwdp;   /* Encrypted password. */\n" +
            "    public Int32 sp_lstchg;      /* Date of last change. */\n" +
            "    public Int32 sp_min;         /* Minimum number of days between changes. */\n" +
            "    public Int32 sp_max;         /* Maximum number of days between changes. */\n" +
            "    public Int32 sp_warn;        /* Number of days to warn user to change the password. */\n" +
            "    public Int32 sp_inact;       /* Number of days the account may be inactive. */\n" +
            "    public Int32 sp_expire;      /* Number of days since 1970-01-01 until account expires. */\n" +
            "    public UInt32 sp_flag;       /* Reserved.  */\n" +
            "\n" +
            "    public Spwd(){\n" +
            "        sp_namp = new AnsiString();\n" +
            "        sp_pwdp = new AnsiString();\n" +
            "        sp_lstchg = new Int32();\n" +
            "        sp_min = new Int32();\n" +
            "        sp_max = new Int32();\n" +
            "        sp_warn = new Int32();\n" +
            "        sp_inact = new Int32();\n" +
            "        sp_expire = new Int32();\n" +
            "        sp_flag = new UInt32();\n" +
            "        initialize(new Argument[]{new Pointer(sp_namp), new Pointer(sp_pwdp), sp_lstchg, sp_min,\n" +
            "                sp_max, sp_warn, sp_inact, sp_expire, sp_flag});\n" +
            "    }\n" +
            "}" +
            "</pre>" +
            "<br>As you may notice creation of structure in Java differs a little from definition of similar structure in C/C++. Thus coding process is simplified repeatedly." +
            "<br>And now lets call getspent() function. Below the typical variant of its usage is presented:" +
            "<br><pre>" +
            "   Library kernel = new Library(\"libc.so\");             /* load libc.so library */\n" +
            "   Function getspent = kernel.getFunction(\"getspent\");  /* locate getspent function */\n" +
            "   Function setspent = kernel.getFunction(\"setspent\");  /* locate setspent function */\n" +
            "   Function endspent = kernel.getFunction(\"endspent\");  /* locate endspent function */\n" +
            "   Spwd spwd = new Spwd();                                /* create Spwd structure */\n" +
            "   Pointer spwdPtr = new Pointer(spwd);                   /* create and initialize pointer to Spwd structure */\n" +
            "   setspent.invoke(null);                                 /* resets the enumeration to the  beginning  of the set of shadow password entries */\n" +
            "   getspent.invoke(spwdPtr);                              /* obtain pointer to the first entry */\n" +
            "   if (!spwdPtr.isNull()){                                /* check if retrieval is successive */\n" +
            "      System.out.println(\"User name: \" + spwd.sp_namp); /* prints user name */\n" +
            "   }\n" +
            "   endspent.invoke(null);                                 /* indicates that no more further shadow password retrieval operations will be done */" +
            "</pre>" +
            "<br>If you require complete information of functions and structures we used, refer man pages." +
            "<p align=right color=black><a href=" + URL_API_REFERENCE + ">View API Reference (in browser)</a></p>" +
            "</p>" +
            "</body></html>";

    private JPanel jPanelOptions = null;
    private JPanel jPanelDesc = null;
    private JPanel jPanelSrc = null;

    private JLabel jLabelFilter;
    private JTextField jTextFieldFilter;
    private JTable jTableUsers;
    private DefaultTableModel usersTabelModel;
    private JTable jTableGroups;
    private DefaultTableModel groupsTableModel;
    private JTabbedPane jTabbedPaneUserGroup;
    private String usersFilterString;
    private String groupsFilterString;


    public String getID() {
        return CONSTRAINT_LIN_USER_MANAGER;
    }

    public String getCaption() {
        return CAPTION_LIN_USER_MANAGER;
    }

    public Vector getContentDescriptions() {
        return null;
    }

    public int getState() {
        return STATE_NONE;
    }

    public JPanel getDemoPanel() {
        JPanel jPanelContents = new JPanel(new GridBagLayout()) {
            public void paint(Graphics g) {
                super.paint(g);
                if (!SysInfoUtil.isUnix() && !SysInfoUtil.isSunOS()) {
                    Font font = g.getFont().deriveFont(Font.BOLD, 20);
                    String str = "To view this demo run it under unix";
                    FontMetrics fontmetrics = getFontMetrics(font);
                    g.setFont(font);
                    g.setColor(Color.black);
                    g.drawString(str, (getWidth() - fontmetrics.stringWidth(str)) / 2, (getHeight() - fontmetrics.getHeight()) / 2);
                }
            }
        };
        TitledBorder titledBorder = BorderFactory.createTitledBorder(CAPTION_LIN_USER_MANAGER);
        titledBorder.setTitleColor(Color.black);
        jPanelContents.setBorder(titledBorder);


        jLabelFilter = new JLabel("User search filter:");
        jLabelFilter.setForeground(Color.black);

        jTextFieldFilter = new JTextField();
        jTextFieldFilter.setForeground(Color.black);
        jTextFieldFilter.getDocument().addDocumentListener(new DocumentListener() {
            public void changedUpdate(DocumentEvent e) {
                processFilterInput();
            }

            public void insertUpdate(DocumentEvent e) {
                processFilterInput();
            }

            public void removeUpdate(DocumentEvent e) {
                processFilterInput();
            }
        });

        jLabelFilter.setPreferredSize(new Dimension(130, jLabelFilter.getPreferredSize().height));
        jLabelFilter.setMinimumSize(new Dimension(130, jLabelFilter.getPreferredSize().height));
        jLabelFilter.setMaximumSize(new Dimension(130, jLabelFilter.getPreferredSize().height));

        Util.constrain(jPanelContents, jLabelFilter,
                0, 0, 1, 1, GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHWEST, 0.0, 0.0,
                5, 5, 5, 5);

        Util.constrain(jPanelContents, jTextFieldFilter,
                1, 0, 1, 1, GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHWEST, 1.0, 0.0,
                5, 5, 5, 5);

        jTabbedPaneUserGroup = new JTabbedPane(JTabbedPane.TOP);
        JPanel usersTab = createUsersTab();
        JPanel groupsTab = createGroupsTab();

        jTabbedPaneUserGroup.addTab("Users", usersTab);
        jTabbedPaneUserGroup.addTab("Groups", groupsTab);
        jTabbedPaneUserGroup.addChangeListener(new ChangeListener() {
            public void stateChanged(ChangeEvent e) {
                switch (jTabbedPaneUserGroup.getSelectedIndex()) {
                    case 0: //users tab
                        jLabelFilter.setText("User search filter:");
                        jTextFieldFilter.setText(usersFilterString);
                        break;
                    case 1: //groups tab
                        jLabelFilter.setText("Group search filter:");
                        jTextFieldFilter.setText(groupsFilterString);

                        break;
                }
            }
        });

        Util.constrain(jPanelContents, new JScrollPane(jTabbedPaneUserGroup),
                0, 1, 3, 1, GridBagConstraints.BOTH, GridBagConstraints.NORTHWEST, 1, 1,
                0, 5, 0, 5);
        return jPanelContents;
    }

    public JPanel getOptionPanel() {
        if (jPanelOptions != null)
            return jPanelOptions;
        return null;
    }

    public JPanel getDescriptionPanel() {
        if (jPanelDesc != null)
            return jPanelDesc;

        jPanelDesc = new JPanel(new GridBagLayout());
        Util.constrain(jPanelDesc, new JLabel(INTRO_LIN_USER_MANAGER),
                0, 1, 1, 1, GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHWEST, 1.0, 1.0,
                5, 5, 5, 5);
        Util.constrain(jPanelDesc, Box.createGlue(),
                0, 2, 1, 1, GridBagConstraints.BOTH, GridBagConstraints.NORTHWEST, 1.0, 1.0,
                0, 0, 0, 0);
        return jPanelDesc;
    }

    public JPanel getSrcPanel() {
        if (jPanelSrc != null)
            return jPanelSrc;

        jPanelSrc = new JPanel(new GridBagLayout());
        jPanelSrc.setBorder(BorderFactory.createTitledBorder("Linux/Solaris: Unix Manual Page for getspent"));

        JEditorPane jEditorPane = new JEditorPane("text/html", SRC_LIN_USER_MANAGER);
        jEditorPane.setEditable(false);
        jEditorPane.setCaretPosition(0);
        jEditorPane.setBackground(jPanelSrc.getBackground());
        jEditorPane.addHyperlinkListener(new HyperlinkListener() {
            public void hyperlinkUpdate(HyperlinkEvent e) {
                if (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
                    if (URL_API_REFERENCE.equals(e.getDescription())) {
                        try {
                            String s;
                            try {
                                s = new File(API_REFERENCE_PATH).getCanonicalFile().toURI().toString();
                            } catch (NoSuchMethodError er) {
                                s = new File(API_REFERENCE_PATH).getCanonicalFile().toURL().toString();
                                s = convertURL2URI(s);
                            }
                            HyperlinkUtil.openInternetBrowser(s);
                        } catch (Exception e1) {
                        }
                    }
                }
            }
        });

        Util.constrain(jPanelSrc, new JScrollPane(jEditorPane),
                0, 1, 1, 1, GridBagConstraints.BOTH, GridBagConstraints.NORTHWEST, 1.0, 1.0,
                5, 5, 5, 5);
        return jPanelSrc;
    }


    public void init(Component parentFrame) {
        // TODO:
//        RefreshHelper.refresh(!SysInfoUtil.isUnix());
        if (SysInfoUtil.isUnix() || SysInfoUtil.isSunOS()) {
            InfoProvider.getInfoProvider().loadInfos();
            GroupManager.getInstance().refresh();
            UserManager.getInstance().refresh();
        }
    }

    public void demoWillBecomeVisible() {
        boolean unixLike = SysInfoUtil.isUnix() || SysInfoUtil.isSunOS();
        jLabelFilter.setEnabled(unixLike);
        jTextFieldFilter.setEnabled(unixLike);
        jTableUsers.setEnabled(unixLike);
        jTableUsers.getTableHeader().setEnabled(unixLike);
        jTableGroups.setEnabled(unixLike);
        jTableGroups.getTableHeader().setEnabled(unixLike);
        jTabbedPaneUserGroup.setEnabled(unixLike);
        refresh();
    }

    private void processFilterInput() {
        switch (jTabbedPaneUserGroup.getSelectedIndex()) {
            case 0: //users tab
                if ((jTextFieldFilter.getText() == null) || "".equals(jTextFieldFilter.getText().trim())) {
                    usersFilterString = "";
                    refreshUsers(null);
                } else {
                    usersFilterString = jTextFieldFilter.getText().trim();
                    refreshUsers(usersFilterString);
                }
                break;
            case 1: //groups tab
                if ((jTextFieldFilter.getText() == null) || "".equals(jTextFieldFilter.getText().trim())) {
                    groupsFilterString = "";
                    refreshGroups(null);
                } else {
                    groupsFilterString = jTextFieldFilter.getText().trim();
                    refreshGroups(groupsFilterString);
                }
                break;
        }
    }

    public void showUserPropertiesDialog(int userId) {
        UserInfo userInfo = UserManager.getInstance().getUserInfo(userId);
        if (userInfo != null) {

            UIUserPropertiesDialog uiUserPropertiesDialog = new UIUserPropertiesDialog((Frame) parentFrame, userInfo, isRoot());
            uiUserPropertiesDialog.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
            uiUserPropertiesDialog.setVisible(true);
        }
    }

    public void showGroupPropertiesDialog(int groupId) {
        GroupInfo groupInfo = GroupManager.getInstance().getGroupInfo(groupId);
        if (groupInfo != null) {
            UIGroupPropertiesDialog uiGroupPropertiesDialog = new UIGroupPropertiesDialog((Frame) parentFrame, groupInfo, isRoot());
            uiGroupPropertiesDialog.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
            uiGroupPropertiesDialog.setVisible(true);
        }
    }

    public void refresh() {
        refreshUsers("");
        refreshGroups("");
    }

    private void refreshUsers(String filter) {
        for (int i = usersTabelModel.getRowCount() - 1; i >= 0; i--) {
            usersTabelModel.removeRow(i);
        }
        UserInfo[] userInfos = UserManager.getInstance().getUserInfos();
        for (int i = 0; i < userInfos.length; i++) {
            UserInfo userInfo = userInfos[i];
            if ((filter == null) || (userInfo.getUserName().startsWith(filter))) {
                Object[] rowData = new Object[jTableUsers.getColumnCount()];
                rowData[0] = userInfo.getUserName();
                rowData[1] = new Integer(userInfo.getId());
                GroupInfo groupInfo = GroupManager.getInstance().getGroupInfo(userInfo.getGroupId());
                if (groupInfo != null) {
                    rowData[2] = groupInfo.getGroupName();
                } else {
                    rowData[2] = GroupInfo.UNKNOWN_GROUP.getGroupName();
                }
                rowData[3] = userInfo.getRealName();
                rowData[4] = userInfo.getShellPrg();
                rowData[5] = userInfo.getHomeDir();
                usersTabelModel.addRow(rowData);
            }
        }
    }

    private void refreshGroups(String filter) {
        for (int i = groupsTableModel.getRowCount() - 1; i >= 0; i--) {
            groupsTableModel.removeRow(i);
        }
        GroupInfo[] groupInfos = GroupManager.getInstance().getGroupInfos();
        for (int i = 0; i < groupInfos.length; i++) {
            GroupInfo grupInfo = groupInfos[i];
            if ((filter == null) || (grupInfo.getGroupName().startsWith(filter))) {
                Object[] rowData = new Object[jTableGroups.getColumnCount()];
                rowData[0] = grupInfo.getGroupName();
                rowData[1] = new Integer(grupInfo.getId());

                StringBuffer buffer = new StringBuffer();
                java.util.List groupMembers = grupInfo.getGroupMembers();
                for (int j = groupMembers.size() - 1; j >= 0; j--) {
                    buffer.append(groupMembers.get(j).toString());
                    if (j != 0) {
                        buffer.append(", ");
                    }
                }
                rowData[2] = buffer.toString();

                groupsTableModel.addRow(rowData);
            }
        }
    }

    private JPanel createUsersTab() {
        JPanel panel = new JPanel(new BorderLayout());
        jTableUsers = createUsersTable();
//        usersTabelModel.addRow(new Object[]{"root", "0", "root", "root", "/bin/bash", "/root"});
//        usersTabelModel.addRow(new Object[]{"root2", "0", "root", "root", "/bin/bash", "/root"});
        JScrollPane scrollPane = new JScrollPane(jTableUsers, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
        panel.add(scrollPane, BorderLayout.CENTER);
        return panel;
    }

    private JPanel createGroupsTab() {
        JPanel panel = new JPanel(new BorderLayout());
        jTableGroups = createGroupsTable();
//        groupsTableModel.addRow(new Object[]{"root", "0", "halt, operator,root"});
//        groupsTableModel.addRow(new Object[]{"root2", "0", "halt, operator,root"});
        JScrollPane scrollPane = new JScrollPane(jTableGroups, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
        panel.add(scrollPane, BorderLayout.CENTER);
        return panel;
    }

    private JTable createUsersTable() {
        usersTabelModel = new DefaultTableModel(new String[]{"User Name", "User ID", "Primary Group", "Full name", "Login Shell", "Home Directory"}, 0) {
            public boolean isCellEditable(int row, int column) {
                return false;
            }
        };
        SortedTableModel sortedTableModel = new SortedTableModel(usersTabelModel);
        final JTable table = new JTable(sortedTableModel) {
            private Color parityColor = new Color(206, 255, 206);
            private Color nonParityColor = new Color(228, 230, 228);

            public TableCellRenderer getCellRenderer(int row, int column) {
                TableCellRenderer cellRenderer = super.getCellRenderer(row, column);
                Component rendererComponent = cellRenderer.getTableCellRendererComponent(jTableUsers, jTableUsers.getValueAt(row, column), jTableUsers.isCellSelected(row, column), false, row, column);
                if (!jTableUsers.isRowSelected(row)) {
                    rendererComponent.setBackground((row % 2 == 0) ? parityColor : nonParityColor);
                }
                return cellRenderer;
            }

            protected void processKeyEvent(KeyEvent e) {
                if ((KeyEvent.VK_ENTER == e.getKeyCode()) && (e.getID() == KeyEvent.KEY_PRESSED)) {
                    int row = jTableUsers.getSelectedRow();
                    if (row != -1) {
                        int uid;
                        try {
                            uid = Integer.parseInt(jTableUsers.getModel().getValueAt(row, 1).toString());
                        } catch (NumberFormatException e1) {
                            e1.printStackTrace();
                            return;
                        }
                        showUserPropertiesDialog(uid);
                        e.consume();
                    }
                } else {
                    super.processKeyEvent(e);
                }
            }
        };
        table.getTableHeader().addMouseListener(new SortedTableHeaderMouseListener(table, sortedTableModel));
        table.getTableHeader().setDefaultRenderer(new SortedTableHeaderRenderer(table, sortedTableModel));

        table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
        table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        table.setCellSelectionEnabled(false);
        table.setRowSelectionAllowed(true);

        table.getColumnModel().getColumn(0).setMinWidth(100);
        table.getColumnModel().getColumn(1).setMinWidth(100);
        table.getColumnModel().getColumn(2).setMinWidth(120);
        table.getTableHeader().setReorderingAllowed(false);
        table.setAutoResizeMode(JTable.AUTO_RESIZE_LAST_COLUMN);


        table.addMouseListener(new MouseAdapter() {
            public void mouseClicked(MouseEvent e) {
                if (e.getClickCount() == 2) {
                    int row = jTableUsers.rowAtPoint(e.getPoint());
                    if (row != -1) {
                        int uid;
                        try {
                            uid = Integer.parseInt(jTableUsers.getModel().getValueAt(row, 1).toString());
                        } catch (NumberFormatException e1) {
                            e1.printStackTrace();
                            return;
                        }
                        showUserPropertiesDialog(uid);
                    }
                }
            }
        });
        return table;
    }

    private JTable createGroupsTable() {
        groupsTableModel = new DefaultTableModel(new String[]{"Group Name", "Group ID", "Group Members"}, 0) {
            public boolean isCellEditable(int row, int column) {
                return false;
            }
        };
        SortedTableModel sortedTableModel = new SortedTableModel(groupsTableModel);
        final JTable table = new JTable(sortedTableModel) {
            private Color parityColor = new Color(206, 255, 206);
            private Color nonParityColor = new Color(228, 230, 228);

            public TableCellRenderer getCellRenderer(int row, int column) {
                TableCellRenderer cellRenderer = super.getCellRenderer(row, column);
                Component rendererComponent = cellRenderer.getTableCellRendererComponent(jTableGroups, jTableGroups.getValueAt(row, column), jTableGroups.isCellSelected(row, column), false, row, column);
                if (!jTableGroups.isRowSelected(row)) {
                    rendererComponent.setBackground((row % 2 == 0) ? parityColor : nonParityColor);
                }
                return cellRenderer;
            }

            protected void processKeyEvent(KeyEvent e) {
                if ((KeyEvent.VK_ENTER == e.getKeyCode()) && (e.getID() == KeyEvent.KEY_PRESSED)) {
                    int row = jTableGroups.getSelectedRow();
                    if (row != -1) {
                        int gid;
                        try {
                            gid = Integer.parseInt(jTableGroups.getModel().getValueAt(row, 1).toString());
                        } catch (NumberFormatException e1) {
                            e1.printStackTrace();
                            return;
                        }
                        showGroupPropertiesDialog(gid);
                        e.consume();
                    }
                } else {
                    super.processKeyEvent(e);
                }
            }
        };
        table.getTableHeader().addMouseListener(new SortedTableHeaderMouseListener(table, sortedTableModel));
        table.getTableHeader().setDefaultRenderer(new SortedTableHeaderRenderer(table, sortedTableModel));

        table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
        table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        table.setCellSelectionEnabled(false);
        table.setRowSelectionAllowed(true);

        table.getColumnModel().getColumn(0).setMinWidth(100);
        table.getColumnModel().getColumn(1).setMinWidth(100);
        table.getColumnModel().getColumn(2).setMinWidth(200);
        table.getTableHeader().setReorderingAllowed(false);
        table.setAutoResizeMode(JTable.AUTO_RESIZE_LAST_COLUMN);

        table.addMouseListener(new MouseAdapter() {
            public void mouseClicked(MouseEvent e) {
                if (e.getClickCount() == 2) {
                    int row = jTableGroups.rowAtPoint(e.getPoint());
                    if (row != -1) {
                        int gid;
                        try {
                            gid = Integer.parseInt(jTableGroups.getModel().getValueAt(row, 1).toString());
                        } catch (NumberFormatException e1) {
                            e1.printStackTrace();
                            return;
                        }
                        showGroupPropertiesDialog(gid);
                    }
                }
            }
        });

        return table;
    }

    public boolean isRoot() {
        return false;
    }
}
