/*
 * This software is the confidential and proprietary information of
 * Smardec. You shall not disclose or distribute such Confidential Information.
 * This software shall not be integrated, copied (partially or fully),
 * or reverse engineered into any other software, program, or code that is
 * not intended for Smardec. You shall use it only in accordance with the terms
 * of the license agreement you entered into with Smardec.
 *
 * Copyright (C) 2004-2006 Smardec. All rights reserved.
 *
 * http://www.smardec.com
 *
 */

package com.smardec.demo.j2native.demos;

import com.smardec.asc.hyperlink.HyperlinkUtil;
import com.smardec.demo.j2native.DemoableAdapter;
import com.smardec.demo.j2native.Util;
import com.smardec.demo.j2native.tools.NativeLibraryHelper;
import com.smardec.demo.j2native.tools.SysInfoUtil;

import javax.swing.*;
import javax.swing.border.TitledBorder;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.Vector;

public class WinSimpleDemo extends DemoableAdapter {
    public final static String CONSTRAINT_WIN_SIMPLE = "WinSimpleDemo";
    public final static String CAPTION_WIN_SIMPLE = "Simple demo";
    public final static String INTRO_WIN_SIMPLE = "<html><head>" + css + "</head><body>" +
            "<p align=left color=black>The simpler, the better  this is the motto of the demonstration. What can be simpler and clearer than invocation of such API functions as  <i>MessageBeep()</i> and <i>MessageBox()</i>. And we will start our immersion into J2Native world from these very functions</p>" +
            "</body></html>";
    public final static String URL_API_REFERENCE = "simple.html";
    public final static String API_REFERENCE_PATH = "demo-resources/api-reference/MessageBox.html";

    public final static String SRC_WIN_SIMPLE = "<html><head>" + css + "</head><body>" +
            "<p align=left color=black>And now lets assume that you should create a simple message and the message window should have only one button OK and an information icon. For this it is enough to do the following in Java application:" +
            "<br><pre>" +
            "    Library library = new Library(\"user32\");                    // load user32.dll library\n" +
            "    Pointer.Void hWnd = new Pointer.Void();                     // we dont care of owner window\n" +
            "                                                                // handle, so set it to NULL\n" +
            "    Pointer pText = new Pointer(new AnsiString(\"Hello World\")); // prepare message text \n" +
            "    Pointer pCaption = new Pointer(new AnsiString(\"Greeting\")); // prepare message box caption\n" +
            "    UInt uFlags = new UInt(MB_OK | MB_ICONINFORMATION);         // set flags \n" +
            "    Function function = library.getFunction(\"MessageBoxA\");     // locate MessageBox ANSI version \n" +
            "    UInt rValue = new UInt();                                   // prepare variable to store result\n" +
            "    function.invoke(rValue, hWnd, pText, pCaption, uFlags);     // invoke function</pre>" +
            "<br>It is easy and the most remarkable thing is that it works." +
            "<p align=right color=black><a href=" + URL_API_REFERENCE + ">View API Reference (in browser)</a></p>" +
            "</p>" +
            "</body></html>";

    private JPanel jPanelOptions = null;
    private JPanel jPanelDesc = null;
    private JPanel jPanelSrc = null;

    private JTextField jTextFieldMessageCaption;
    private JTextArea jTextAreaMessage;
    private JComboBox jComboBoxMessageIcon;
    private JComboBox jComboBoxMessageStyle;
    private JComboBox jComboBoxTone;
    private JButton jButtonBeep;
    private JButton jButtonMessage;

    public String getID() {
        return CONSTRAINT_WIN_SIMPLE;
    }

    public String getCaption() {
        return CAPTION_WIN_SIMPLE;
    }

    public Vector getContentDescriptions() {
        return null;
    }

    public int getState() {
        return STATE_NONE;
    }

    public JPanel getDemoPanel() {
        JPanel jPanelContents = new JPanel(new GridBagLayout()) {
            public void paint(Graphics g) {
                super.paint(g);
                if (!SysInfoUtil.isWin()) {
                    Font font = g.getFont().deriveFont(Font.BOLD, 20);
                    String str = "To view this demo run it under windows";
                    FontMetrics fontmetrics = getFontMetrics(font);
                    g.setFont(font);
                    g.setColor(Color.black);
                    g.drawString(str, (getWidth() - fontmetrics.stringWidth(str)) / 2, (getHeight() - fontmetrics.getHeight()) / 2);
                }
            }
        };
        TitledBorder titledBorder = BorderFactory.createTitledBorder("Simple library method invocation demonstation");
        titledBorder.setTitleColor(Color.black);
        jPanelContents.setBorder(titledBorder);

        JPanel messageBeepPanel = new JPanel(new GridBagLayout());
        titledBorder = BorderFactory.createTitledBorder("I. User32.dll :: MessageBeep()");
        titledBorder.setTitleColor(Color.black);
        messageBeepPanel.setBorder(titledBorder);

        JLabel toneLabel = new JLabel("Tone:");
        toneLabel.setForeground(Color.black);
        Util.constrain(messageBeepPanel, toneLabel,
                0, 0, 1, 1, GridBagConstraints.NONE, GridBagConstraints.EAST, 0.0, 0.0,
                5, 5, 5, 5);

        jComboBoxTone = new JComboBox(new Object[]{"MB_OK", "MB_ICONHAND", "MB_ICONQUESTION", "MB_ICONEXCLAMATION", "MB_ICONASTERISK"});
        Util.constrain(messageBeepPanel, jComboBoxTone,
                1, 0, 2, 1, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST, 1.0, 0.0,
                5, 5, 5, 5);

        jButtonBeep = new JButton("Invoke");
        jButtonBeep.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                NativeLibraryHelper.getInstance().messageBeepWin(jComboBoxTone.getSelectedIndex() << 4);
            }
        });
        Util.constrain(messageBeepPanel, jButtonBeep,
                2, 1, 1, 1, GridBagConstraints.NONE, GridBagConstraints.EAST, 0.0, 0.0,
                5, 5, 5, 5);


        JPanel messageBoxPanel = new JPanel(new GridBagLayout());
        titledBorder = BorderFactory.createTitledBorder("II. User32.dll :: MessageBox()");
        titledBorder.setTitleColor(Color.black);
        messageBoxPanel.setBorder(titledBorder);

        JLabel captionLabel = new JLabel("Caption:");
        captionLabel.setForeground(Color.black);
        Util.constrain(messageBoxPanel, captionLabel,
                0, 0, 1, 1, GridBagConstraints.NONE, GridBagConstraints.EAST, 0.0, 0.0,
                5, 5, 5, 5);

        jTextFieldMessageCaption = new JTextField();
        jTextFieldMessageCaption.setText("Caption");
        Util.constrain(messageBoxPanel, jTextFieldMessageCaption,
                1, 0, 3, 1, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST, 1.0, 0.0,
                5, 5, 5, 5);

        JLabel textLabel = new JLabel("Text:");
        textLabel.setForeground(Color.black);
        Util.constrain(messageBoxPanel, textLabel,
                0, 1, 1, 1, GridBagConstraints.NONE, GridBagConstraints.EAST, 0.0, 0.0,
                5, 5, 5, 5);

        jTextAreaMessage = new JTextArea();
        jTextAreaMessage.setText("Text");
        JScrollPane textScroll = new JScrollPane(jTextAreaMessage);
        Util.constrain(messageBoxPanel, textScroll,
                1, 1, 3, 3, GridBagConstraints.BOTH, GridBagConstraints.WEST, 1.0, 1.0,
                5, 5, 5, 5);

        JLabel iconLabel = new JLabel("Icon:");
        iconLabel.setForeground(Color.black);
        Util.constrain(messageBoxPanel, iconLabel,
                0, 4, 1, 1, GridBagConstraints.NONE, GridBagConstraints.EAST, 0.0, 0.0,
                5, 5, 5, 5);

        jComboBoxMessageIcon = new JComboBox(new Object[]{"MB_OK", "MB_ICONHAND", "MB_ICONQUESTION", "MB_ICONEXCLAMATION", "MB_ICONASTERISK"});
        Util.constrain(messageBoxPanel, jComboBoxMessageIcon,
                1, 4, 1, 1, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST, 0.5, 0.0,
                5, 5, 5, 5);

        JLabel styleLabel = new JLabel("Style:");
        styleLabel.setForeground(Color.black);
        Util.constrain(messageBoxPanel, styleLabel,
                2, 4, 1, 1, GridBagConstraints.NONE, GridBagConstraints.EAST, 0.0, 0.0,
                5, 10, 5, 5);

        jComboBoxMessageStyle = new JComboBox(new Object[]{"MB_OK", "MB_OKCANCEL", "MB_ABORTRETRYIGNORE", "MB_YESNOCANCEL",
                "MB_YESNO", "MB_RETRYCANCEL"});
        Util.constrain(messageBoxPanel, jComboBoxMessageStyle,
                3, 4, 1, 1, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST, 0.5, 0.0,
                5, 5, 5, 5);

        jButtonMessage = new JButton("Invoke");
        jButtonMessage.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                new Thread() {
                    public void run() {
                        NativeLibraryHelper.getInstance().showMessageBoxWin(jTextFieldMessageCaption.getText(),
                                jTextAreaMessage.getText(),
                                jComboBoxMessageStyle.getSelectedIndex() | (jComboBoxMessageIcon.getSelectedIndex() << 4));
                    }
                }.start();
            }
        });
        Util.constrain(messageBoxPanel, jButtonMessage,
                3, 5, 1, 1, GridBagConstraints.NONE, GridBagConstraints.EAST, 0.0, 0.0,
                5, 5, 5, 5);

        Util.constrain(jPanelContents, messageBeepPanel,
                0, 0, 1, 1, GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHWEST, 1, 0,
                0, 5, 0, 5);
        Util.constrain(jPanelContents, messageBoxPanel,
                0, 1, 1, 1, GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHWEST, 1, 1,
                10, 5, 0, 5);
        return jPanelContents;
    }

    public JPanel getOptionPanel() {
        if (jPanelOptions != null)
            return jPanelOptions;
        return null;
    }

    public JPanel getDescriptionPanel() {
        if (jPanelDesc != null)
            return jPanelDesc;

        jPanelDesc = new JPanel(new GridBagLayout());
        Util.constrain(jPanelDesc, new JLabel(INTRO_WIN_SIMPLE),
                0, 1, 1, 1, GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHWEST, 1.0, 1.0,
                5, 5, 5, 5);
        Util.constrain(jPanelDesc, Box.createGlue(),
                0, 2, 1, 1, GridBagConstraints.BOTH, GridBagConstraints.NORTHWEST, 1.0, 1.0,
                0, 0, 0, 0);
        return jPanelDesc;
    }

    public JPanel getSrcPanel() {
        if (jPanelSrc != null)
            return jPanelSrc;

        jPanelSrc = new JPanel(new GridBagLayout());
        jPanelSrc.setBorder(BorderFactory.createTitledBorder("Windows: Simple Windows API Invocation Code Sample"));

        JEditorPane jEditorPane = new JEditorPane("text/html", SRC_WIN_SIMPLE);
        jEditorPane.setEditable(false);
        jEditorPane.setCaretPosition(0);
        jEditorPane.setBackground(jPanelSrc.getBackground());
        jEditorPane.addHyperlinkListener(new HyperlinkListener() {
            public void hyperlinkUpdate(HyperlinkEvent e) {
                if (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
                    if (URL_API_REFERENCE.equals(e.getDescription())) {
                        try {
                            String s;
                            try {
                                s = new File(API_REFERENCE_PATH).getCanonicalFile().toURI().toString();
                            } catch (NoSuchMethodError er) {
                                s = new File(API_REFERENCE_PATH).getCanonicalFile().toURL().toString();
                                s = convertURL2URI(s);
                            }
                            HyperlinkUtil.openInternetBrowser(s);
                        } catch (Exception e1) {
                        }
                    }
                }
            }
        });

        Util.constrain(jPanelSrc, new JScrollPane(jEditorPane),
                0, 1, 1, 1, GridBagConstraints.BOTH, GridBagConstraints.NORTHWEST, 1.0, 1.0,
                5, 5, 5, 5);
        return jPanelSrc;
    }

    public void demoWillBecomeVisible() {
        boolean win = SysInfoUtil.isWin();
        jTextFieldMessageCaption.setEnabled(win);
        jTextAreaMessage.setEnabled(win);
        jComboBoxMessageIcon.setEnabled(win);
        jComboBoxMessageStyle.setEnabled(win);
        jComboBoxTone.setEnabled(win);
        jButtonBeep.setEnabled(win);
        jButtonMessage.setEnabled(win);
    }
}
