/*
 * This software is the confidential and proprietary information of
 * Smardec. You shall not disclose or distribute such Confidential Information.
 * This software shall not be integrated, copied (partially or fully),
 * or reverse engineered into any other software, program, or code that is
 * not intended for Smardec. You shall use it only in accordance with the terms
 * of the license agreement you entered into with Smardec.
 *
 * Copyright (C) 2004-2006 Smardec. All rights reserved.
 *
 * http://www.smardec.com
 *
 */

package com.smardec.demo.j2native.win32.cdplayer;

import com.smardec.demo.j2native.tools.LibraryManager;
import com.smardec.j2native.*;

public class HWndBtn {
    private static final int stopIndex = 0;
    private static final int prevIndex = 2;
    private static final int playIndex = 4;
    private static final int nextIndex = 8;
    private static final int doorIndex = 10;

    private CDPlayer cdPlayer;

    public Pointer.Void stop;
    public Pointer.Void prev;
    public Pointer.Void play;
    public Pointer.Void next;
    public Pointer.Void door;
    public Pointer.Void oldButtonProc;
    public NewButtonProc newButtonProc;

    public HWndBtn(CDPlayer cdPlayer) {
        this.cdPlayer = cdPlayer;
        stop = new Pointer.Void();
        prev = new Pointer.Void();
        play = new Pointer.Void();
        next = new Pointer.Void();
        door = new Pointer.Void();
        oldButtonProc = new Pointer.Void();
        newButtonProc = new NewButtonProc();
    }

    class NewButtonProc extends Callback {
        private Pointer.Void hWnd = new Pointer.Void();
        private UInt32 uMsg = new UInt32();
        private UInt32 wParam = new UInt32();
        private Int32 lParam = new Int32();
        private UInt32 lResult = new UInt32();

        private Function callWindowProc;
        private Function beginPaint;
        private Function imageList_Draw;
        private Function setCapture;
        private Function getDC;
        private Function releaseDC;
        private Function releaseCapture;
        private Function endPaint;
        private Function postMessage;

        private Pointer.Void hdc = new Pointer.Void();
        private Pointer ps = new Pointer(new PAINTSTRUCT());

        public NewButtonProc() {

            try {
                Library user32 = LibraryManager.getInstance().getLibrary("user32");
                Library comctl32 = LibraryManager.getInstance().getLibrary("comctl32");

                callWindowProc = user32.getFunction("CallWindowProcA");
                beginPaint = user32.getFunction("BeginPaint");
                imageList_Draw = comctl32.getFunction("ImageList_Draw");
                setCapture = user32.getFunction("SetCapture");
                getDC = user32.getFunction("GetDC");
                releaseDC = user32.getFunction("ReleaseDC");
                releaseCapture = user32.getFunction("ReleaseCapture");
                endPaint = user32.getFunction("EndPaint");
                postMessage = user32.getFunction("PostMessageA");
            } catch (FunctionNotFoundException e) {
                e.printStackTrace();
                return;
            }

            initialize(lResult, new Argument[]{hWnd, uMsg, wParam, lParam});
        }

        public void callback() {
            int index;
            Pointer.Void imageListInUse = cdPlayer.getHImageList();
            if (hWnd.equals(stop)) {
                index = HWndBtn.stopIndex;
            } else if (hWnd.equals(prev)) {
                index = HWndBtn.prevIndex;
            } else if (hWnd.equals(play)) {
                index = (cdPlayer.playing) ? HWndBtn.playIndex + 2 : HWndBtn.playIndex;
            } else if (hWnd.equals(next)) {
                index = HWndBtn.nextIndex;
            } else if (hWnd.equals(door)) {
                index = 0;//HWndBtn.doorIndex;
                imageListInUse = cdPlayer.getHImageListLarge();
            } else {
                index = -1;
            }

            switch (uMsg.intValue()) {
                case CDPConstants.WM_PAINT:
                    beginPaint.invoke(hdc, hWnd, ps);
                    imageList_Draw.invoke(null, new Argument[]{imageListInUse, new Int(index), hdc, new Int(0),
                            new Int(0), new UInt(CDPConstants.ILD_TRANSPARENT)});
                    endPaint.invoke(null, hWnd, ps);
                    lResult.setValue(0);
                    return;
                case CDPConstants.WM_LBUTTONDOWN:
                    setCapture.invoke(null, hWnd);
                    getDC.invoke(hdc, hWnd);
                    imageList_Draw.invoke(null, new Argument[]{imageListInUse, new Int(index + 1), hdc, new Int(0),
                            new Int(0), new UInt(CDPConstants.ILD_TRANSPARENT)});
                    releaseDC.invoke(null, hWnd, hdc);
                    lResult.setValue(0);
                    return;
                case CDPConstants.WM_LBUTTONUP:
                    releaseCapture.invoke(null);
                    getDC.invoke(hdc, hWnd);
                    int t = index;
                    if (index == (HWndBtn.playIndex + 2)) {
                        t -= 2;
                    }
                    if ((index == HWndBtn.playIndex) && (!cdPlayer.noDisc)) {
                        t += 2;
                    }
                    imageList_Draw.invoke(null, new Argument[]{imageListInUse, new Int(t), hdc, new Int(0),
                            new Int(0), new UInt(CDPConstants.ILD_TRANSPARENT)});
                    releaseDC.invoke(null, hWnd, hdc);
                    if (hWnd.equals(door)) {
                        index = HWndBtn.doorIndex;
                    }
                    postMessage.invoke(null, cdPlayer.getHWndMain(), new UInt(CDPConstants.WM_COMMAND),
                            new Int((index == HWndBtn.playIndex + 2) ? index - 2 : index), hWnd);
                    lResult.setValue(0);
                    return;
                case CDPConstants.WM_LBUTTONDBLCLK:
                    lResult.setValue(0);
                    return;
            }
            callWindowProc.invoke(lResult, new Argument[]{oldButtonProc, hWnd, uMsg, wParam, lParam});

/*
  int Index;
  if (hWnd==hWndBtn.Stop) Index=StopIndex;
  else if(hWnd==hWndBtn.Prev) Index=PrevIndex;
  else if(hWnd==hWndBtn.Play) Index=PlayIndex;
  else if(hWnd==hWndBtn.Next) Index=NextIndex;
  else if(hWnd==hWndBtn.Door) Index=DoorIndex;
  if ((Index==PlayIndex)&&(CD.Playing)) Index+=2;

  switch(uMsg){
    case WM_PAINT:
      HDC hDC;
                PAINTSTRUCT PS;

      hDC=BeginPaint(hWnd,&PS);
      ImageList_Draw(hImageList,Index,hDC,0,0,ILD_TRANSPARENT);
      EndPaint(hWnd,&PS);
      return 0;
    case WM_LBUTTONDOWN:
      SetCapture(hWnd);
      hDC=GetDC(hWnd);
      ImageList_Draw(hImageList,Index+1,hDC,0,0,ILD_TRANSPARENT);
      ReleaseDC(hWnd,hDC);
      return 0;
    case WM_LBUTTONUP:
      ReleaseCapture();
      hDC=GetDC(hWnd);
      int t;t=Index;
      if(Index==PlayIndex+2) t-=2;
      if((Index==PlayIndex)&&(!CD.NoDisc)) t+=2;
      ImageList_Draw(hImageList,t,hDC,0,0,ILD_TRANSPARENT);
      ReleaseDC(hWnd,hDC);
      PostMessage(hWndMain,WM_COMMAND,Index==PlayIndex+2? Index-2 : Index,
                  (LONG)hWnd);//Index==Button's ID
      return 0;
    case WM_LBUTTONDBLCLK:
      return 0;
  }
  return CallWindowProc(hWndBtn.OldButtonProc,hWnd,uMsg,wParam,lParam);
*/
        }
    }

}

/*
class _hWndBtn{
public:
  HWND Stop,Prev,Play,Next,Door;
  #ifdef STRICT
    WNDPROC OldButtonProc;
  #else
    FARPROC OldButtonProc;
  #endif

  static LRESULT CALLBACK NewButtonProc(HWND hWnd,UINT uMsg,WPARAM wParam,LPARAM lParam);
  // static function is used to get pointer: in C++ you cannot get
  // pointer to non-static member function
}hWndBtn;
*/